@@ -1,26 +1,22 @@
 package Net::DNS::Parameters;
 
-#
-# $Id: Parameters.pm 1727 2018-12-31 12:04:48Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1727 $)[1];
-
-
 ################################################
 ##
 ##	Domain Name System (DNS) Parameters
-##	(last updated 2018-12-12)
+##	(last updated 2020-12-02)
 ##
 ################################################
 
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: Parameters.pm 1827 2020-12-14 10:49:27Z willem $)[2];
+
 use integer;
 use Carp;
 
 use base qw(Exporter);
-our @EXPORT = qw(
+
+our @EXPORT_OK = qw(
 		classbyname classbyval %classbyname
 		typebyname typebyval %typebyname
 		opcodebyname opcodebyval
@@ -29,6 +25,15 @@ our @EXPORT = qw(
 		dsotypebyname dsotypebyval
 		);
 
+our %EXPORT_TAGS = (
+	class	   => [qw(classbyname classbyval)],
+	type	   => [qw(typebyname typebyval)],
+	opcode	   => [qw(opcodebyname opcodebyval)],
+	rcode	   => [qw(rcodebyname rcodebyval)],
+	ednsoption => [qw(ednsoptionbyname ednsoptionbyval)],
+	dsotype	   => [qw(dsotypebyname dsotypebyval)],
+	);
+
 
 # Registry: DNS CLASSes
 my @classbyname = (
@@ -39,7 +44,7 @@ my @classbyname = (
 	ANY  => 255,						# RFC1035
 	);
 our %classbyval = reverse( CLASS0 => 0, @classbyname );
-push @classbyname, map /^\d/ ? $_ : lc($_), @classbyname;
+push @classbyname, map { /^\d/ ? $_ : lc($_) } @classbyname;
 our %classbyname = ( '*' => 255, @classbyname );
 
 
@@ -106,7 +111,9 @@ my @typebyname = (
 	CDNSKEY	   => 60,					# RFC7344
 	OPENPGPKEY => 61,					# RFC7929
 	CSYNC	   => 62,					# RFC7477
-	ZONEMD	   => 63,					# draft-wessels-dns-zone-digest
+	ZONEMD	   => 63,					# RFC-ietf-dnsop-dns-zone-digest-14
+	SVCB	   => 64,					# draft-ietf-dnsop-svcb-https-00
+	HTTPS	   => 65,					# draft-ietf-dnsop-svcb-https-00
 	SPF	   => 99,					# RFC7208
 	UINFO	   => 100,					# IANA-Reserved
 	UID	   => 101,					# IANA-Reserved
@@ -119,21 +126,22 @@ my @typebyname = (
 	EUI48	   => 108,					# RFC7043
 	EUI64	   => 109,					# RFC7043
 	TKEY	   => 249,					# RFC2930
-	TSIG	   => 250,					# RFC2845
+	TSIG	   => 250,					# RFC8945
 	IXFR	   => 251,					# RFC1995
 	AXFR	   => 252,					# RFC1035 RFC5936
 	MAILB	   => 253,					# RFC1035
 	MAILA	   => 254,					# RFC1035
-	ANY	   => 255,					# RFC1035 RFC6895 RFC-ietf-dnsop-refuse-any-07
+	ANY	   => 255,					# RFC1035 RFC6895 RFC8482
 	URI	   => 256,					# RFC7553
-	CAA	   => 257,					# RFC6844
+	CAA	   => 257,					# RFC8659
 	AVC	   => 258,					#
 	DOA	   => 259,					# draft-durand-doa-over-dns
+	AMTRELAY   => 260,					# RFC8777
 	TA	   => 32768,					# http://cameo.library.cmu.edu/ http://www.watson.org/~weiler/INI1999-19.pdf
-	DLV	   => 32769,					# RFC4431
+	DLV	   => 32769,					# RFC8749 RFC4431
 	);
 our %typebyval = reverse( TYPE0 => 0, @typebyname );
-push @typebyname, map /^\d/ ? $_ : lc($_), @typebyname;
+push @typebyname, map { /^\d/ ? $_ : lc($_) } @typebyname;
 our %typebyname = ( '*' => 255, @typebyname );
 
 
@@ -144,10 +152,10 @@ my @opcodebyname = (
 	STATUS => 2,						# RFC1035
 	NOTIFY => 4,						# RFC1996
 	UPDATE => 5,						# RFC2136
-	DSO    => 6,						# RFC-ietf-dnsop-session-signal-20
+	DSO    => 6,						# RFC8490
 	);
 our %opcodebyval = reverse @opcodebyname;
-push @opcodebyname, map /^\d/ ? $_ : lc($_), @opcodebyname;
+push @opcodebyname, map { /^\d/ ? $_ : lc($_) } @opcodebyname;
 our %opcodebyname = ( NS_NOTIFY_OP => 4, @opcodebyname );
 
 
@@ -163,43 +171,46 @@ my @rcodebyname = (
 	YXRRSET	  => 7,						# RFC2136
 	NXRRSET	  => 8,						# RFC2136
 	NOTAUTH	  => 9,						# RFC2136
-	NOTAUTH	  => 9,						# RFC2845
+	NOTAUTH	  => 9,						# RFC8945
 	NOTZONE	  => 10,					# RFC2136
-	DSOTYPENI => 11,					# RFC-ietf-dnsop-session-signal-20
+	DSOTYPENI => 11,					# RFC8490
 	BADVERS	  => 16,					# RFC6891
-	BADSIG	  => 16,					# RFC2845
-	BADKEY	  => 17,					# RFC2845
-	BADTIME	  => 18,					# RFC2845
+	BADSIG	  => 16,					# RFC8945
+	BADKEY	  => 17,					# RFC8945
+	BADTIME	  => 18,					# RFC8945
 	BADMODE	  => 19,					# RFC2930
 	BADNAME	  => 20,					# RFC2930
 	BADALG	  => 21,					# RFC2930
-	BADTRUNC  => 22,					# RFC4635
+	BADTRUNC  => 22,					# RFC8945
 	BADCOOKIE => 23,					# RFC7873
 	);
 our %rcodebyval = reverse( BADSIG => 16, @rcodebyname );
-push @rcodebyname, map /^\d/ ? $_ : lc($_), @rcodebyname;
+push @rcodebyname, map { /^\d/ ? $_ : lc($_) } @rcodebyname;
 our %rcodebyname = @rcodebyname;
 
 
 # Registry: DNS EDNS0 Option Codes (OPT)
 my @ednsoptionbyname = (
-	LLQ		=> 1,					# http://files.dns-sd.org/draft-sekar-dns-llq.txt
-	UL		=> 2,					# http://files.dns-sd.org/draft-sekar-dns-ul.txt
-	NSID		=> 3,					# RFC5001
-	DAU		=> 5,					# RFC6975
-	DHU		=> 6,					# RFC6975
-	N3U		=> 7,					# RFC6975
-	'CLIENT-SUBNET' => 8,					# RFC7871
-	EXPIRE		=> 9,					# RFC7314
-	COOKIE		=> 10,					# RFC7873
-	'TCP-KEEPALIVE' => 11,					# RFC7828
-	PADDING		=> 12,					# RFC7830
-	CHAIN		=> 13,					# RFC7901
-	'KEY-TAG'	=> 14,					# RFC8145
-	DEVICEID	=> 26946,				# https://docs.umbrella.com/developer/networkdevices-api/identifying-dns-traffic2
+	LLQ		 => 1,					# RFC8764
+	UL		 => 2,					# http://files.dns-sd.org/draft-sekar-dns-ul.txt
+	NSID		 => 3,					# RFC5001
+	DAU		 => 5,					# RFC6975
+	DHU		 => 6,					# RFC6975
+	N3U		 => 7,					# RFC6975
+	'CLIENT-SUBNET'	 => 8,					# RFC7871
+	EXPIRE		 => 9,					# RFC7314
+	COOKIE		 => 10,					# RFC7873
+	'TCP-KEEPALIVE'	 => 11,					# RFC7828
+	PADDING		 => 12,					# RFC7830
+	CHAIN		 => 13,					# RFC7901
+	'KEY-TAG'	 => 14,					# RFC8145
+	'EXTENDED-ERROR' => 15,					# RFC8914
+	'CLIENT-TAG'	 => 16,					# draft-bellis-dnsop-edns-tags
+	'SERVER-TAG'	 => 17,					# draft-bellis-dnsop-edns-tags
+	DEVICEID	 => 26946,				# https://docs.umbrella.com/developer/networkdevices-api/identifying-dns-traffic2
 	);
 our %ednsoptionbyval = reverse @ednsoptionbyname;
-push @ednsoptionbyname, map /^\d/ ? $_ : lc($_), @ednsoptionbyname;
+push @ednsoptionbyname, map { /^\d/ ? $_ : lc($_) } @ednsoptionbyname;
 our %ednsoptionbyname = @ednsoptionbyname;
 
 
@@ -212,7 +223,7 @@ my @dnsflagbyname = (
 	AD => 0x0020,						# RFC4035 RFC6840
 	CD => 0x0010,						# RFC4035 RFC6840
 	);
-push @dnsflagbyname, map /^\d/ ? $_ : lc($_), @dnsflagbyname;
+push @dnsflagbyname, map { /^\d/ ? $_ : lc($_) } @dnsflagbyname;
 our %dnsflagbyname = @dnsflagbyname;
 
 
@@ -220,18 +231,22 @@ our %dnsflagbyname = @dnsflagbyname;
 my @ednsflagbyname = (
 	DO => 0x8000,						# RFC4035 RFC3225 RFC6840
 	);
-push @ednsflagbyname, map /^\d/ ? $_ : lc($_), @ednsflagbyname;
+push @ednsflagbyname, map { /^\d/ ? $_ : lc($_) } @ednsflagbyname;
 our %ednsflagbyname = @ednsflagbyname;
 
 
 # Registry: DSO Type Codes
 my @dsotypebyname = (
-	KEEPALIVE	  => 0x0001,				# RFC-ietf-dnsop-session-signal-20
-	RETRYDELAY	  => 0x0002,				# RFC-ietf-dnsop-session-signal-20
-	ENCRYPTIONPADDING => 0x0003,				# RFC-ietf-dnsop-session-signal-20
+	KEEPALIVE	  => 0x0001,				# RFC8490
+	RETRYDELAY	  => 0x0002,				# RFC8490
+	ENCRYPTIONPADDING => 0x0003,				# RFC8490
+	SUBSCRIBE	  => 0x0040,				# RFC8765
+	PUSH		  => 0x0041,				# RFC8765
+	UNSUBSCRIBE	  => 0x0042,				# RFC8765
+	RECONFIRM	  => 0x0043,				# RFC8765
 	);
 our %dsotypebyval = reverse @dsotypebyname;
-push @dsotypebyname, map /^\d/ ? $_ : lc($_), @dsotypebyname;
+push @dsotypebyname, map { /^\d/ ? $_ : lc($_) } @dsotypebyname;
 our %dsotypebyname = @dsotypebyname;
 
 
@@ -242,29 +257,29 @@ our %dsotypebyname = @dsotypebyname;
 sub classbyname {
 	my $name = shift;
 
-	$classbyname{$name} || $classbyname{uc $name} || do {
+	return $classbyname{$name} || $classbyname{uc $name} || return do {
 		croak qq[unknown class "$name"] unless $name =~ m/^(CLASS)?(\d+)/i;
 		my $val = 0 + $2;
 		croak qq[classbyname("$name") out of range] if $val > 0xffff;
 		return $val;
-			}
+	}
 }
 
 sub classbyval {
 	my $val = shift;
 
-	$classbyval{$val} || do {
+	return $classbyval{$val} || return do {
 		$val += 0;
 		croak qq[classbyval($val) out of range] if $val > 0xffff;
 		return "CLASS$val";
-			}
+	}
 }
 
 
 sub typebyname {
 	my $name = shift;
 
-	$typebyname{$name} || do {
+	return $typebyname{$name} || return do {
 		if ( $name =~ m/^(TYPE)?(\d+)/i ) {
 			my $val = 0 + $2;
 			croak qq[typebyname("$name") out of range] if $val > 0xffff;
@@ -272,80 +287,84 @@ sub typebyname {
 		}
 		_typespec("$name.RRNAME") unless $typebyname{uc $name};
 		return $typebyname{uc $name} || croak qq[unknown type "$name"];
-			}
+	}
 }
 
 sub typebyval {
 	my $val = shift;
 
-	$typebyval{$val} || do {
+	return $typebyval{$val} || return do {
 		$val += 0;
 		croak qq[typebyval($val) out of range] if $val > 0xffff;
 		$typebyval{$val} = "TYPE$val";
 		_typespec("$val.RRTYPE");
 		return $typebyval{$val};
-			}
+	}
 }
 
 
 sub opcodebyname {
 	my $arg = shift;
-	return $opcodebyname{$arg} if defined $opcodebyname{$arg};
-	return 0 + $arg if $arg =~ /^\d/;
+	my $val = $opcodebyname{$arg};
+	return $val if defined $val;
+	return $arg if $arg =~ /^\d/;
 	croak qq[unknown opcode "$arg"];
 }
 
 sub opcodebyval {
 	my $val = shift;
-	$opcodebyval{$val} || return $val;
+	return $opcodebyval{$val} || return "$val";
 }
 
 
 sub rcodebyname {
 	my $arg = shift;
-	return $rcodebyname{$arg} if defined $rcodebyname{$arg};
-	return 0 + $arg if $arg =~ /^\d/;
+	my $val = $rcodebyname{$arg};
+	return $val if defined $val;
+	return $arg if $arg =~ /^\d/;
 	croak qq[unknown rcode "$arg"];
 }
 
 sub rcodebyval {
 	my $val = shift;
-	$rcodebyval{$val} || return $val;
+	return $rcodebyval{$val} || return "$val";
 }
 
 
 sub ednsoptionbyname {
 	my $arg = shift;
-	return $ednsoptionbyname{$arg} if defined $ednsoptionbyname{$arg};
-	return 0 + $arg if $arg =~ /^\d/;
+	my $val = $ednsoptionbyname{$arg};
+	return $val if defined $val;
+	return $arg if $arg =~ /^\d/;
 	croak qq[unknown option "$arg"];
 }
 
 sub ednsoptionbyval {
 	my $val = shift;
-	$ednsoptionbyval{$val} || return $val;
+	return $ednsoptionbyval{$val} || return "$val";
 }
 
 
 sub dsotypebyname {
 	my $arg = shift;
-	return $dsotypebyname{$arg} if defined $dsotypebyname{$arg};
-	return 0 + $arg if $arg =~ /^\d/;
+	my $val = $dsotypebyname{$arg};
+	return $val if defined $val;
+	return $arg if $arg =~ /^\d/;
 	croak qq[unknown DSO type "$arg"];
 }
 
 sub dsotypebyval {
 	my $val = shift;
-	$dsotypebyval{$val} || return $val;
+	return $dsotypebyval{$val} || return "$val";
 }
 
 
 sub register {				## register( 'TOY', 1234 )	(NOT part of published API)
-	my ( $mnemonic, $rrtype ) = map uc($_), @_;		# uncoverable pod
+	my ( $mnemonic, $rrtype ) = @_;				# uncoverable pod
 	$rrtype = rand(255) + 65280 unless $rrtype;
+	croak qq["$mnemonic" is a CLASS identifier] if defined $classbyname{$mnemonic = uc($mnemonic)};
 	for ( typebyval( $rrtype = int $rrtype ) ) {
-		croak qq["$mnemonic" is a CLASS identifier] if $classbyname{$mnemonic};
-		return $rrtype if /^$mnemonic$/;    # duplicate registration
+		return $rrtype if /^$mnemonic$/;		# duplicate registration
 		croak qq["$mnemonic" conflicts with TYPE$rrtype ($_)] unless /^TYPE\d+$/;
 		my $known = $typebyname{$mnemonic};
 		croak qq["$mnemonic" conflicts with TYPE$known] if $known;
@@ -355,32 +374,33 @@ sub register {				## register( 'TOY', 1234 )	(NOT part of published API)
 }
 
 
-use constant EXTLANG => defined eval 'require Net::DNS::Extlang';
+use constant EXTLANG => defined eval { require Net::DNS::Extlang };
 
-our $DNSEXTLANG = EXTLANG ? eval 'Net::DNS::Extlang->new()->domain' : undef;
+our $DNSEXTLANG = EXTLANG ? eval { Net::DNS::Extlang->new()->domain } : undef;
 
-sub _typespec {				## draft-levine-dnsextlang
-	eval <<'END' if EXTLANG && $DNSEXTLANG;
+sub _typespec {
+	eval {				## draft-levine-dnsextlang
+		<<'END' } if EXTLANG && $DNSEXTLANG;
 	my ($node) = @_;
 
 	require Net::DNS::Resolver;
-	my $resolver = new Net::DNS::Resolver() || return;
+	my $resolver = Net::DNS::Resolver->new() || return;
 	my $response = $resolver->send( "$node.$DNSEXTLANG", 'TXT' ) || return;
 
-	foreach my $txt ( grep $_->type eq 'TXT', $response->answer ) {
+	foreach my $txt ( grep { $_->type eq 'TXT' } $response->answer ) {
 		my @stanza = $txt->txtdata;
 		my ( $tag, $identifier, @attribute ) = @stanza;
 		next unless defined($tag) && $tag =~ /^RRTYPE=\d+$/;
 		register( $1, $2 ) if $identifier =~ /^(\w+):(\d+)\W*/;
 		return unless defined wantarray;
 
-		my $extobj = new Net::DNS::Extlang();
+		my $extobj = Net::DNS::Extlang->new();
 		my $recipe = $extobj->xlstorerecord( $identifier, @attribute );
 		my @source = split /\n/, $extobj->compilerr($recipe);
 		return sub { defined( $_ = shift @source ) };
 	}
-	return;
 END
+	return;
 }
 
 
