@@ -1,4 +1,4 @@
-# $Id: NonFatal.pm 1816 2020-10-16 09:44:21Z willem $	-*-perl-*-
+# $Id: NonFatal.pm 1827 2020-12-14 10:49:27Z willem $	-*-perl-*-
 
 # Test::More calls functions from Test::Builder. Those functions all eventually
 # call Test::Builder::ok (on a builder instance) for reporting the status.
@@ -6,9 +6,9 @@
 # ok method that always reports the test to have completed successfully.
 #
 # The functions NonFatalBegin and NonFatalEnd re-bless the builder in use by
-# Test::More (Test::More->builder) to be of type Test::NonFatal and
-# Test::Builder respectively. Tests that are between those functions will thus
-# always appear to succeed. The failure report itself is not suppressed.
+# Test::More (Test::More->builder) to be of type NonFatal and Test::Builder
+# respectively. Tests that are between those functions will thus always appear
+# to succeed. The failure report itself is not suppressed.
 #
 # Note that the builder is only re-blessed when the file 't/online.nonfatal'
 # exists.
@@ -18,7 +18,7 @@
 # NonFatalEnd subroutines may not be nested.
 #
 
-package Test::NonFatal;
+package NonFatal;
 
 use strict;
 use warnings;
@@ -33,23 +33,15 @@ sub ok {
 
 	$self->SUPER::ok( 1, "NOT OK (tolerating failure)  @name" );
 
-	push @failed, scalar(@name) ? "@name" : 'undef';
+	push @failed, join( "\t", $self->current_test, @name );
 	return $test;
 }
 
 
-sub diag {
-	my @annotation = @_;
-	return Test::More->builder->diag(@annotation);
-}
-
-
 END {
-	my $n = scalar(@failed);
-	my $s = $n > 1 ? 's' : '';
-	bless Test::More->builder, qw(Test::Builder);
-	diag( join "\n\t", "\tDisregarding $n failed sub-test$s", @failed ) if $n;
-	return;
+	my $n = scalar(@failed) || return;
+	my $s = ( $n == 1 ) ? '' : 's';
+	Test::Builder->new->diag( join "\n\t", "\tDisregarding $n failed sub-test$s", @failed );
 }
 
 
@@ -60,7 +52,7 @@ require Test::More;
 use constant NONFATAL => eval { -e 't/online.nonfatal' };
 
 sub NonFatalBegin {
-	bless Test::More->builder, qw(Test::NonFatal) if NONFATAL;
+	bless Test::More->builder, qw(NonFatal) if NONFATAL;
 	return;
 }
 
